#include "fortran.def"
c========================================================================
c/////////////////////// SUBROUTINE POP3_LUMINOSITY \\\\\\\\\\\\\\\\\\\\\
c
      subroutine pop3_properties(mass, luminosity, lifetime)
c
c     FINDS POPULATION III STELLAR PROPERTIES FROM TABLES
c
c     written by: John Wise
c     date:       12 September 2005
c
c INPUTS:
c     mass - stellar mass [in solar masses]
c
c OUTPUTS:
c     luminosity - [in solar units]
c     lifetime - [years]
c
c------------------------------------------------------------------------
      implicit none
#include "fortran_types.def"
c------------------------------------------------------------------------

      R_PREC mass, luminosity, lifetime

      R_PREC minmass, maxmass
      parameter (minmass=5, maxmass=1000)

      R_PREC lumtable(996), lifetable(996)
      
      data lumtable
     $    /7.413e2_RKIND,1.257e3_RKIND,2.094e3_RKIND,3.364e3_RKIND,
     $     5.117e3_RKIND,7.274e3_RKIND,9.725e3_RKIND,1.237e4_RKIND,
     $     1.513e4_RKIND,1.801e4_RKIND,2.109e4_RKIND,2.452e4_RKIND,
     $     2.838e4_RKIND,3.269e4_RKIND,3.747e4_RKIND,4.276e4_RKIND,
     $     4.859e4_RKIND,5.496e4_RKIND,6.192e4_RKIND,6.947e4_RKIND,
     $     7.762e4_RKIND,8.641e4_RKIND,9.581e4_RKIND,1.058e5_RKIND,
     $     1.165e5_RKIND,1.277e5_RKIND,1.395e5_RKIND,1.518e5_RKIND,
     $     1.646e5_RKIND,1.779e5_RKIND,1.915e5_RKIND,2.055e5_RKIND,
     $     2.197e5_RKIND,2.341e5_RKIND,2.486e5_RKIND,2.630e5_RKIND,
     $     2.774e5_RKIND,2.917e5_RKIND,3.057e5_RKIND,3.196e5_RKIND,
     $     3.332e5_RKIND,3.466e5_RKIND,3.597e5_RKIND,3.725e5_RKIND,
     $     3.851e5_RKIND,3.975e5_RKIND,4.096e5_RKIND,4.216e5_RKIND,
     $     4.335e5_RKIND,4.453e5_RKIND,4.571e5_RKIND,4.689e5_RKIND,
     $     4.809e5_RKIND,4.932e5_RKIND,5.058e5_RKIND,5.188e5_RKIND,
     $     5.324e5_RKIND,5.465e5_RKIND,5.612e5_RKIND,5.765e5_RKIND,
     $     5.923e5_RKIND,6.086e5_RKIND,6.255e5_RKIND,6.429e5_RKIND,
     $     6.609e5_RKIND,6.793e5_RKIND,6.983e5_RKIND,7.176e5_RKIND,
     $     7.374e5_RKIND,7.576e5_RKIND,7.782e5_RKIND,7.991e5_RKIND,
     $     8.203e5_RKIND,8.417e5_RKIND,8.634e5_RKIND,8.851e5_RKIND,
     $     9.069e5_RKIND,9.288e5_RKIND,9.508e5_RKIND,9.728e5_RKIND,
     $     9.948e5_RKIND,1.017e6_RKIND,1.039e6_RKIND,1.061e6_RKIND,
     $     1.083e6_RKIND,1.105e6_RKIND,1.127e6_RKIND,1.149e6_RKIND,
     $     1.171e6_RKIND,1.193e6_RKIND,1.215e6_RKIND,1.236e6_RKIND,
     $     1.258e6_RKIND,1.280e6_RKIND,1.301e6_RKIND,1.323e6_RKIND,
     $     1.345e6_RKIND,1.366e6_RKIND,1.388e6_RKIND,1.409e6_RKIND,
     $     1.430e6_RKIND,1.452e6_RKIND,1.473e6_RKIND,1.494e6_RKIND,
     $     1.515e6_RKIND,1.537e6_RKIND,1.558e6_RKIND,1.579e6_RKIND,
     $     1.600e6_RKIND,1.621e6_RKIND,1.643e6_RKIND,1.664e6_RKIND,
     $     1.685e6_RKIND,1.707e6_RKIND,1.728e6_RKIND,1.750e6_RKIND,
     $     1.772e6_RKIND,1.793e6_RKIND,1.815e6_RKIND,1.837e6_RKIND,
     $     1.860e6_RKIND,1.882e6_RKIND,1.904e6_RKIND,1.927e6_RKIND,
     $     1.949e6_RKIND,1.972e6_RKIND,1.995e6_RKIND,2.018e6_RKIND,
     $     2.041e6_RKIND,2.064e6_RKIND,2.087e6_RKIND,2.111e6_RKIND,
     $     2.134e6_RKIND,2.158e6_RKIND,2.181e6_RKIND,2.205e6_RKIND,
     $     2.229e6_RKIND,2.253e6_RKIND,2.276e6_RKIND,2.301e6_RKIND,
     $     2.325e6_RKIND,2.349e6_RKIND,2.373e6_RKIND,2.398e6_RKIND,
     $     2.422e6_RKIND,2.447e6_RKIND,2.471e6_RKIND,2.496e6_RKIND,
     $     2.521e6_RKIND,2.545e6_RKIND,2.570e6_RKIND,2.595e6_RKIND,
     $     2.620e6_RKIND,2.645e6_RKIND,2.671e6_RKIND,2.696e6_RKIND,
     $     2.721e6_RKIND,2.747e6_RKIND,2.772e6_RKIND,2.798e6_RKIND,
     $     2.823e6_RKIND,2.849e6_RKIND,2.874e6_RKIND,2.900e6_RKIND,
     $     2.926e6_RKIND,2.952e6_RKIND,2.978e6_RKIND,3.004e6_RKIND,
     $     3.030e6_RKIND,3.056e6_RKIND,3.082e6_RKIND,3.108e6_RKIND,
     $     3.134e6_RKIND,3.160e6_RKIND,3.187e6_RKIND,3.213e6_RKIND,
     $     3.240e6_RKIND,3.266e6_RKIND,3.292e6_RKIND,3.319e6_RKIND,
     $     3.346e6_RKIND,3.372e6_RKIND,3.399e6_RKIND,3.426e6_RKIND,
     $     3.452e6_RKIND,3.479e6_RKIND,3.506e6_RKIND,3.533e6_RKIND,
     $     3.560e6_RKIND,3.587e6_RKIND,3.614e6_RKIND,3.641e6_RKIND,
     $     3.668e6_RKIND,3.695e6_RKIND,3.722e6_RKIND,3.750e6_RKIND,
     $     3.777e6_RKIND,3.804e6_RKIND,3.832e6_RKIND,3.859e6_RKIND,
     $     3.887e6_RKIND,3.914e6_RKIND,3.942e6_RKIND,3.969e6_RKIND,
     $     3.997e6_RKIND,4.024e6_RKIND,4.052e6_RKIND,4.080e6_RKIND,
     $     4.107e6_RKIND,4.135e6_RKIND,4.163e6_RKIND,4.191e6_RKIND,
     $     4.219e6_RKIND,4.246e6_RKIND,4.274e6_RKIND,4.302e6_RKIND,
     $     4.330e6_RKIND,4.358e6_RKIND,4.386e6_RKIND,4.414e6_RKIND,
     $     4.442e6_RKIND,4.470e6_RKIND,4.498e6_RKIND,4.527e6_RKIND,
     $     4.555e6_RKIND,4.583e6_RKIND,4.611e6_RKIND,4.639e6_RKIND,
     $     4.667e6_RKIND,4.696e6_RKIND,4.724e6_RKIND,4.752e6_RKIND,
     $     4.781e6_RKIND,4.809e6_RKIND,4.837e6_RKIND,4.865e6_RKIND,
     $     4.894e6_RKIND,4.922e6_RKIND,4.951e6_RKIND,4.979e6_RKIND,
     $     5.007e6_RKIND,5.036e6_RKIND,5.064e6_RKIND,5.093e6_RKIND,
     $     5.121e6_RKIND,5.150e6_RKIND,5.178e6_RKIND,5.207e6_RKIND,
     $     5.235e6_RKIND,5.264e6_RKIND,5.292e6_RKIND,5.321e6_RKIND,
     $     5.349e6_RKIND,5.378e6_RKIND,5.406e6_RKIND,5.435e6_RKIND,
     $     5.464e6_RKIND,5.492e6_RKIND,5.521e6_RKIND,5.550e6_RKIND,
     $     5.578e6_RKIND,5.607e6_RKIND,5.636e6_RKIND,5.664e6_RKIND,
     $     5.693e6_RKIND,5.722e6_RKIND,5.750e6_RKIND,5.779e6_RKIND,
     $     5.808e6_RKIND,5.837e6_RKIND,5.865e6_RKIND,5.894e6_RKIND,
     $     5.923e6_RKIND,5.952e6_RKIND,5.981e6_RKIND,6.010e6_RKIND,
     $     6.039e6_RKIND,6.067e6_RKIND,6.096e6_RKIND,6.125e6_RKIND,
     $     6.154e6_RKIND,6.183e6_RKIND,6.212e6_RKIND,6.241e6_RKIND,
     $     6.270e6_RKIND,6.299e6_RKIND,6.329e6_RKIND,6.358e6_RKIND,
     $     6.387e6_RKIND,6.416e6_RKIND,6.445e6_RKIND,6.474e6_RKIND,
     $     6.504e6_RKIND,6.533e6_RKIND,6.562e6_RKIND,6.592e6_RKIND,
     $     6.621e6_RKIND,6.651e6_RKIND,6.680e6_RKIND,6.709e6_RKIND,
     $     6.739e6_RKIND,6.769e6_RKIND,6.798e6_RKIND,6.828e6_RKIND,
     $     6.857e6_RKIND,6.887e6_RKIND,6.917e6_RKIND,6.947e6_RKIND,
     $     6.976e6_RKIND,7.006e6_RKIND,7.036e6_RKIND,7.066e6_RKIND,
     $     7.096e6_RKIND,7.126e6_RKIND,7.155e6_RKIND,7.185e6_RKIND,
     $     7.215e6_RKIND,7.245e6_RKIND,7.275e6_RKIND,7.305e6_RKIND,
     $     7.336e6_RKIND,7.366e6_RKIND,7.396e6_RKIND,7.426e6_RKIND,
     $     7.456e6_RKIND,7.486e6_RKIND,7.516e6_RKIND,7.547e6_RKIND,
     $     7.577e6_RKIND,7.607e6_RKIND,7.638e6_RKIND,7.668e6_RKIND,
     $     7.698e6_RKIND,7.729e6_RKIND,7.759e6_RKIND,7.789e6_RKIND,
     $     7.820e6_RKIND,7.850e6_RKIND,7.881e6_RKIND,7.911e6_RKIND,
     $     7.942e6_RKIND,7.972e6_RKIND,8.003e6_RKIND,8.033e6_RKIND,
     $     8.064e6_RKIND,8.094e6_RKIND,8.125e6_RKIND,8.155e6_RKIND,
     $     8.186e6_RKIND,8.217e6_RKIND,8.247e6_RKIND,8.278e6_RKIND,
     $     8.309e6_RKIND,8.339e6_RKIND,8.370e6_RKIND,8.401e6_RKIND,
     $     8.432e6_RKIND,8.462e6_RKIND,8.493e6_RKIND,8.524e6_RKIND,
     $     8.555e6_RKIND,8.585e6_RKIND,8.616e6_RKIND,8.647e6_RKIND,
     $     8.678e6_RKIND,8.709e6_RKIND,8.740e6_RKIND,8.771e6_RKIND,
     $     8.801e6_RKIND,8.832e6_RKIND,8.863e6_RKIND,8.894e6_RKIND,
     $     8.925e6_RKIND,8.956e6_RKIND,8.987e6_RKIND,9.018e6_RKIND,
     $     9.049e6_RKIND,9.080e6_RKIND,9.111e6_RKIND,9.142e6_RKIND,
     $     9.173e6_RKIND,9.204e6_RKIND,9.235e6_RKIND,9.266e6_RKIND,
     $     9.297e6_RKIND,9.328e6_RKIND,9.359e6_RKIND,9.390e6_RKIND,
     $     9.421e6_RKIND,9.452e6_RKIND,9.483e6_RKIND,9.514e6_RKIND,
     $     9.545e6_RKIND,9.576e6_RKIND,9.607e6_RKIND,9.638e6_RKIND,
     $     9.669e6_RKIND,9.700e6_RKIND,9.732e6_RKIND,9.763e6_RKIND,
     $     9.794e6_RKIND,9.825e6_RKIND,9.856e6_RKIND,9.887e6_RKIND,
     $     9.918e6_RKIND,9.950e6_RKIND,9.981e6_RKIND,1.001e7_RKIND,
     $     1.004e7_RKIND,1.007e7_RKIND,1.011e7_RKIND,1.014e7_RKIND,
     $     1.017e7_RKIND,1.020e7_RKIND,1.023e7_RKIND,1.026e7_RKIND,
     $     1.029e7_RKIND,1.032e7_RKIND,1.035e7_RKIND,1.039e7_RKIND,
     $     1.042e7_RKIND,1.045e7_RKIND,1.048e7_RKIND,1.051e7_RKIND,
     $     1.054e7_RKIND,1.057e7_RKIND,1.060e7_RKIND,1.064e7_RKIND,
     $     1.067e7_RKIND,1.070e7_RKIND,1.073e7_RKIND,1.076e7_RKIND,
     $     1.079e7_RKIND,1.082e7_RKIND,1.085e7_RKIND,1.089e7_RKIND,
     $     1.092e7_RKIND,1.095e7_RKIND,1.098e7_RKIND,1.101e7_RKIND,
     $     1.104e7_RKIND,1.107e7_RKIND,1.110e7_RKIND,1.114e7_RKIND,
     $     1.117e7_RKIND,1.120e7_RKIND,1.123e7_RKIND,1.126e7_RKIND,
     $     1.129e7_RKIND,1.132e7_RKIND,1.135e7_RKIND,1.139e7_RKIND,
     $     1.142e7_RKIND,1.145e7_RKIND,1.148e7_RKIND,1.151e7_RKIND,
     $     1.154e7_RKIND,1.157e7_RKIND,1.160e7_RKIND,1.164e7_RKIND,
     $     1.167e7_RKIND,1.170e7_RKIND,1.173e7_RKIND,1.176e7_RKIND,
     $     1.179e7_RKIND,1.182e7_RKIND,1.185e7_RKIND,1.189e7_RKIND,
     $     1.192e7_RKIND,1.195e7_RKIND,1.198e7_RKIND,1.201e7_RKIND,
     $     1.204e7_RKIND,1.207e7_RKIND,1.211e7_RKIND,1.214e7_RKIND,
     $     1.217e7_RKIND,1.220e7_RKIND,1.223e7_RKIND,1.226e7_RKIND,
     $     1.229e7_RKIND,1.232e7_RKIND,1.236e7_RKIND,1.239e7_RKIND,
     $     1.242e7_RKIND,1.245e7_RKIND,1.248e7_RKIND,1.251e7_RKIND,
     $     1.254e7_RKIND,1.258e7_RKIND,1.261e7_RKIND,1.264e7_RKIND,
     $     1.267e7_RKIND,1.270e7_RKIND,1.273e7_RKIND,1.276e7_RKIND,
     $     1.280e7_RKIND,1.283e7_RKIND,1.286e7_RKIND,1.289e7_RKIND,
     $     1.292e7_RKIND,1.295e7_RKIND,1.298e7_RKIND,1.302e7_RKIND,
     $     1.305e7_RKIND,1.308e7_RKIND,1.311e7_RKIND,1.314e7_RKIND,
     $     1.317e7_RKIND,1.321e7_RKIND,1.324e7_RKIND,1.327e7_RKIND,
     $     1.330e7_RKIND,1.333e7_RKIND,1.336e7_RKIND,1.339e7_RKIND,
     $     1.343e7_RKIND,1.346e7_RKIND,1.349e7_RKIND,1.352e7_RKIND,
     $     1.355e7_RKIND,1.358e7_RKIND,1.362e7_RKIND,1.365e7_RKIND,
     $     1.368e7_RKIND,1.371e7_RKIND,1.374e7_RKIND,1.377e7_RKIND,
     $     1.381e7_RKIND,1.384e7_RKIND,1.387e7_RKIND,1.390e7_RKIND,
     $     1.393e7_RKIND,1.396e7_RKIND,1.400e7_RKIND,1.403e7_RKIND,
     $     1.406e7_RKIND,1.409e7_RKIND,1.412e7_RKIND,1.415e7_RKIND,
     $     1.419e7_RKIND,1.422e7_RKIND,1.425e7_RKIND,1.428e7_RKIND,
     $     1.431e7_RKIND,1.434e7_RKIND,1.438e7_RKIND,1.441e7_RKIND,
     $     1.444e7_RKIND,1.447e7_RKIND,1.450e7_RKIND,1.453e7_RKIND,
     $     1.456e7_RKIND,1.460e7_RKIND,1.463e7_RKIND,1.466e7_RKIND,
     $     1.469e7_RKIND,1.472e7_RKIND,1.475e7_RKIND,1.479e7_RKIND,
     $     1.482e7_RKIND,1.485e7_RKIND,1.488e7_RKIND,1.491e7_RKIND,
     $     1.494e7_RKIND,1.498e7_RKIND,1.501e7_RKIND,1.504e7_RKIND,
     $     1.507e7_RKIND,1.510e7_RKIND,1.513e7_RKIND,1.517e7_RKIND,
     $     1.520e7_RKIND,1.523e7_RKIND,1.526e7_RKIND,1.529e7_RKIND,
     $     1.532e7_RKIND,1.535e7_RKIND,1.539e7_RKIND,1.542e7_RKIND,
     $     1.545e7_RKIND,1.548e7_RKIND,1.551e7_RKIND,1.554e7_RKIND,
     $     1.558e7_RKIND,1.561e7_RKIND,1.564e7_RKIND,1.567e7_RKIND,
     $     1.570e7_RKIND,1.573e7_RKIND,1.576e7_RKIND,1.580e7_RKIND,
     $     1.583e7_RKIND,1.586e7_RKIND,1.589e7_RKIND,1.592e7_RKIND,
     $     1.595e7_RKIND,1.598e7_RKIND,1.602e7_RKIND,1.605e7_RKIND,
     $     1.608e7_RKIND,1.611e7_RKIND,1.614e7_RKIND,1.617e7_RKIND,
     $     1.620e7_RKIND,1.624e7_RKIND,1.627e7_RKIND,1.630e7_RKIND,
     $     1.633e7_RKIND,1.636e7_RKIND,1.639e7_RKIND,1.642e7_RKIND,
     $     1.645e7_RKIND,1.649e7_RKIND,1.652e7_RKIND,1.655e7_RKIND,
     $     1.658e7_RKIND,1.661e7_RKIND,1.664e7_RKIND,1.667e7_RKIND,
     $     1.670e7_RKIND,1.674e7_RKIND,1.677e7_RKIND,1.680e7_RKIND,
     $     1.683e7_RKIND,1.686e7_RKIND,1.689e7_RKIND,1.692e7_RKIND,
     $     1.695e7_RKIND,1.699e7_RKIND,1.702e7_RKIND,1.705e7_RKIND,
     $     1.708e7_RKIND,1.711e7_RKIND,1.714e7_RKIND,1.717e7_RKIND,
     $     1.720e7_RKIND,1.723e7_RKIND,1.727e7_RKIND,1.730e7_RKIND,
     $     1.733e7_RKIND,1.736e7_RKIND,1.739e7_RKIND,1.742e7_RKIND,
     $     1.745e7_RKIND,1.748e7_RKIND,1.751e7_RKIND,1.754e7_RKIND,
     $     1.757e7_RKIND,1.761e7_RKIND,1.764e7_RKIND,1.767e7_RKIND,
     $     1.770e7_RKIND,1.773e7_RKIND,1.776e7_RKIND,1.779e7_RKIND,
     $     1.782e7_RKIND,1.785e7_RKIND,1.788e7_RKIND,1.791e7_RKIND,
     $     1.794e7_RKIND,1.798e7_RKIND,1.801e7_RKIND,1.804e7_RKIND,
     $     1.807e7_RKIND,1.810e7_RKIND,1.813e7_RKIND,1.816e7_RKIND,
     $     1.819e7_RKIND,1.822e7_RKIND,1.825e7_RKIND,1.828e7_RKIND,
     $     1.831e7_RKIND,1.834e7_RKIND,1.837e7_RKIND,1.840e7_RKIND,
     $     1.844e7_RKIND,1.847e7_RKIND,1.850e7_RKIND,1.853e7_RKIND,
     $     1.856e7_RKIND,1.859e7_RKIND,1.862e7_RKIND,1.865e7_RKIND,
     $     1.868e7_RKIND,1.871e7_RKIND,1.874e7_RKIND,1.877e7_RKIND,
     $     1.880e7_RKIND,1.883e7_RKIND,1.886e7_RKIND,1.889e7_RKIND,
     $     1.892e7_RKIND,1.895e7_RKIND,1.898e7_RKIND,1.901e7_RKIND,
     $     1.904e7_RKIND,1.907e7_RKIND,1.910e7_RKIND,1.913e7_RKIND,
     $     1.916e7_RKIND,1.919e7_RKIND,1.922e7_RKIND,1.925e7_RKIND,
     $     1.929e7_RKIND,1.932e7_RKIND,1.935e7_RKIND,1.938e7_RKIND,
     $     1.941e7_RKIND,1.944e7_RKIND,1.947e7_RKIND,1.950e7_RKIND,
     $     1.953e7_RKIND,1.956e7_RKIND,1.959e7_RKIND,1.962e7_RKIND,
     $     1.965e7_RKIND,1.968e7_RKIND,1.971e7_RKIND,1.974e7_RKIND,
     $     1.977e7_RKIND,1.980e7_RKIND,1.983e7_RKIND,1.986e7_RKIND,
     $     1.989e7_RKIND,1.992e7_RKIND,1.995e7_RKIND,1.998e7_RKIND,
     $     2.000e7_RKIND,2.003e7_RKIND,2.006e7_RKIND,2.009e7_RKIND,
     $     2.012e7_RKIND,2.015e7_RKIND,2.018e7_RKIND,2.021e7_RKIND,
     $     2.024e7_RKIND,2.027e7_RKIND,2.030e7_RKIND,2.033e7_RKIND,
     $     2.036e7_RKIND,2.039e7_RKIND,2.042e7_RKIND,2.045e7_RKIND,
     $     2.048e7_RKIND,2.051e7_RKIND,2.054e7_RKIND,2.057e7_RKIND,
     $     2.060e7_RKIND,2.063e7_RKIND,2.066e7_RKIND,2.069e7_RKIND,
     $     2.072e7_RKIND,2.075e7_RKIND,2.078e7_RKIND,2.081e7_RKIND,
     $     2.083e7_RKIND,2.086e7_RKIND,2.089e7_RKIND,2.092e7_RKIND,
     $     2.095e7_RKIND,2.098e7_RKIND,2.101e7_RKIND,2.104e7_RKIND,
     $     2.107e7_RKIND,2.110e7_RKIND,2.113e7_RKIND,2.116e7_RKIND,
     $     2.119e7_RKIND,2.122e7_RKIND,2.125e7_RKIND,2.128e7_RKIND,
     $     2.130e7_RKIND,2.133e7_RKIND,2.136e7_RKIND,2.139e7_RKIND,
     $     2.142e7_RKIND,2.145e7_RKIND,2.148e7_RKIND,2.151e7_RKIND,
     $     2.154e7_RKIND,2.157e7_RKIND,2.160e7_RKIND,2.163e7_RKIND,
     $     2.165e7_RKIND,2.168e7_RKIND,2.171e7_RKIND,2.174e7_RKIND,
     $     2.177e7_RKIND,2.180e7_RKIND,2.183e7_RKIND,2.186e7_RKIND,
     $     2.189e7_RKIND,2.192e7_RKIND,2.195e7_RKIND,2.197e7_RKIND,
     $     2.200e7_RKIND,2.203e7_RKIND,2.206e7_RKIND,2.209e7_RKIND,
     $     2.212e7_RKIND,2.215e7_RKIND,2.218e7_RKIND,2.221e7_RKIND,
     $     2.224e7_RKIND,2.226e7_RKIND,2.229e7_RKIND,2.232e7_RKIND,
     $     2.235e7_RKIND,2.238e7_RKIND,2.241e7_RKIND,2.244e7_RKIND,
     $     2.247e7_RKIND,2.250e7_RKIND,2.252e7_RKIND,2.255e7_RKIND,
     $     2.258e7_RKIND,2.261e7_RKIND,2.264e7_RKIND,2.267e7_RKIND,
     $     2.270e7_RKIND,2.273e7_RKIND,2.275e7_RKIND,2.278e7_RKIND,
     $     2.281e7_RKIND,2.284e7_RKIND,2.287e7_RKIND,2.290e7_RKIND,
     $     2.293e7_RKIND,2.296e7_RKIND,2.299e7_RKIND,2.301e7_RKIND,
     $     2.304e7_RKIND,2.307e7_RKIND,2.310e7_RKIND,2.313e7_RKIND,
     $     2.316e7_RKIND,2.319e7_RKIND,2.322e7_RKIND,2.324e7_RKIND,
     $     2.327e7_RKIND,2.330e7_RKIND,2.333e7_RKIND,2.336e7_RKIND,
     $     2.339e7_RKIND,2.342e7_RKIND,2.344e7_RKIND,2.347e7_RKIND,
     $     2.350e7_RKIND,2.353e7_RKIND,2.356e7_RKIND,2.359e7_RKIND,
     $     2.362e7_RKIND,2.365e7_RKIND,2.367e7_RKIND,2.370e7_RKIND,
     $     2.373e7_RKIND,2.376e7_RKIND,2.379e7_RKIND,2.382e7_RKIND,
     $     2.385e7_RKIND,2.387e7_RKIND,2.390e7_RKIND,2.393e7_RKIND,
     $     2.396e7_RKIND,2.399e7_RKIND,2.402e7_RKIND,2.405e7_RKIND,
     $     2.408e7_RKIND,2.410e7_RKIND,2.413e7_RKIND,2.416e7_RKIND,
     $     2.419e7_RKIND,2.422e7_RKIND,2.425e7_RKIND,2.428e7_RKIND,
     $     2.430e7_RKIND,2.433e7_RKIND,2.436e7_RKIND,2.439e7_RKIND,
     $     2.442e7_RKIND,2.445e7_RKIND,2.448e7_RKIND,2.451e7_RKIND,
     $     2.453e7_RKIND,2.456e7_RKIND,2.459e7_RKIND,2.462e7_RKIND,
     $     2.465e7_RKIND,2.468e7_RKIND,2.471e7_RKIND,2.474e7_RKIND,
     $     2.476e7_RKIND,2.479e7_RKIND,2.482e7_RKIND,2.485e7_RKIND,
     $     2.488e7_RKIND,2.491e7_RKIND,2.494e7_RKIND,2.497e7_RKIND,
     $     2.499e7_RKIND,2.502e7_RKIND,2.505e7_RKIND,2.508e7_RKIND,
     $     2.511e7_RKIND,2.514e7_RKIND,2.517e7_RKIND,2.520e7_RKIND,
     $     2.522e7_RKIND,2.525e7_RKIND,2.528e7_RKIND,2.531e7_RKIND,
     $     2.534e7_RKIND,2.537e7_RKIND,2.540e7_RKIND,2.543e7_RKIND,
     $     2.546e7_RKIND,2.548e7_RKIND,2.551e7_RKIND,2.554e7_RKIND,
     $     2.557e7_RKIND,2.560e7_RKIND,2.563e7_RKIND,2.566e7_RKIND,
     $     2.569e7_RKIND,2.572e7_RKIND,2.575e7_RKIND,2.577e7_RKIND,
     $     2.580e7_RKIND,2.583e7_RKIND,2.586e7_RKIND,2.589e7_RKIND,
     $     2.592e7_RKIND,2.595e7_RKIND,2.598e7_RKIND,2.601e7_RKIND,
     $     2.604e7_RKIND,2.607e7_RKIND,2.610e7_RKIND,2.612e7_RKIND,
     $     2.615e7_RKIND,2.618e7_RKIND,2.621e7_RKIND,2.624e7_RKIND,
     $     2.627e7_RKIND,2.630e7_RKIND,2.633e7_RKIND,2.636e7_RKIND,
     $     2.639e7_RKIND,2.642e7_RKIND,2.645e7_RKIND,2.648e7_RKIND,
     $     2.651e7_RKIND,2.654e7_RKIND,2.657e7_RKIND,2.659e7_RKIND,
     $     2.662e7_RKIND,2.665e7_RKIND,2.668e7_RKIND,2.671e7_RKIND,
     $     2.674e7_RKIND,2.677e7_RKIND,2.680e7_RKIND,2.683e7_RKIND,
     $     2.686e7_RKIND,2.689e7_RKIND,2.692e7_RKIND,2.695e7_RKIND,
     $     2.698e7_RKIND,2.701e7_RKIND,2.704e7_RKIND,2.707e7_RKIND,
     $     2.710e7_RKIND,2.713e7_RKIND,2.716e7_RKIND,2.719e7_RKIND,
     $     2.722e7_RKIND,2.725e7_RKIND,2.728e7_RKIND,2.731e7_RKIND,
     $     2.734e7_RKIND,2.737e7_RKIND,2.740e7_RKIND,2.743e7_RKIND,
     $     2.746e7_RKIND,2.749e7_RKIND,2.752e7_RKIND,2.755e7_RKIND,
     $     2.758e7_RKIND,2.761e7_RKIND,2.764e7_RKIND,2.767e7_RKIND,
     $     2.771e7_RKIND,2.774e7_RKIND,2.777e7_RKIND,2.780e7_RKIND/

      data lifetable
     $    /6.190e7_RKIND,4.536e7_RKIND,3.366e7_RKIND,2.560e7_RKIND,
     $     2.022e7_RKIND,1.673e7_RKIND,1.443e7_RKIND,1.288e7_RKIND,
     $     1.181e7_RKIND,1.102e7_RKIND,1.040e7_RKIND,9.853e6_RKIND,
     $     9.357e6_RKIND,8.907e6_RKIND,8.494e6_RKIND,8.114e6_RKIND,
     $     7.762e6_RKIND,7.432e6_RKIND,7.122e6_RKIND,6.829e6_RKIND,
     $     6.549e6_RKIND,6.281e6_RKIND,6.024e6_RKIND,5.779e6_RKIND,
     $     5.548e6_RKIND,5.329e6_RKIND,5.124e6_RKIND,4.931e6_RKIND,
     $     4.752e6_RKIND,4.587e6_RKIND,4.434e6_RKIND,4.294e6_RKIND,
     $     4.168e6_RKIND,4.054e6_RKIND,3.953e6_RKIND,3.864e6_RKIND,
     $     3.788e6_RKIND,3.723e6_RKIND,3.668e6_RKIND,3.622e6_RKIND,
     $     3.585e6_RKIND,3.554e6_RKIND,3.529e6_RKIND,3.510e6_RKIND,
     $     3.496e6_RKIND,3.486e6_RKIND,3.480e6_RKIND,3.476e6_RKIND,
     $     3.474e6_RKIND,3.473e6_RKIND,3.473e6_RKIND,3.474e6_RKIND,
     $     3.474e6_RKIND,3.472e6_RKIND,3.469e6_RKIND,3.464e6_RKIND,
     $     3.456e6_RKIND,3.444e6_RKIND,3.431e6_RKIND,3.414e6_RKIND,
     $     3.396e6_RKIND,3.376e6_RKIND,3.354e6_RKIND,3.331e6_RKIND,
     $     3.306e6_RKIND,3.280e6_RKIND,3.254e6_RKIND,3.227e6_RKIND,
     $     3.199e6_RKIND,3.172e6_RKIND,3.144e6_RKIND,3.117e6_RKIND,
     $     3.090e6_RKIND,3.063e6_RKIND,3.037e6_RKIND,3.012e6_RKIND,
     $     2.988e6_RKIND,2.965e6_RKIND,2.943e6_RKIND,2.921e6_RKIND,
     $     2.901e6_RKIND,2.881e6_RKIND,2.863e6_RKIND,2.844e6_RKIND,
     $     2.827e6_RKIND,2.811e6_RKIND,2.795e6_RKIND,2.779e6_RKIND,
     $     2.765e6_RKIND,2.751e6_RKIND,2.737e6_RKIND,2.724e6_RKIND,
     $     2.712e6_RKIND,2.700e6_RKIND,2.688e6_RKIND,2.677e6_RKIND,
     $     2.667e6_RKIND,2.656e6_RKIND,2.647e6_RKIND,2.637e6_RKIND,
     $     2.628e6_RKIND,2.619e6_RKIND,2.611e6_RKIND,2.603e6_RKIND,
     $     2.595e6_RKIND,2.587e6_RKIND,2.580e6_RKIND,2.573e6_RKIND,
     $     2.566e6_RKIND,2.559e6_RKIND,2.552e6_RKIND,2.546e6_RKIND,
     $     2.539e6_RKIND,2.533e6_RKIND,2.527e6_RKIND,2.521e6_RKIND,
     $     2.515e6_RKIND,2.509e6_RKIND,2.503e6_RKIND,2.498e6_RKIND,
     $     2.492e6_RKIND,2.486e6_RKIND,2.481e6_RKIND,2.475e6_RKIND,
     $     2.470e6_RKIND,2.464e6_RKIND,2.459e6_RKIND,2.454e6_RKIND,
     $     2.448e6_RKIND,2.443e6_RKIND,2.438e6_RKIND,2.433e6_RKIND,
     $     2.428e6_RKIND,2.423e6_RKIND,2.418e6_RKIND,2.414e6_RKIND,
     $     2.409e6_RKIND,2.404e6_RKIND,2.400e6_RKIND,2.395e6_RKIND,
     $     2.390e6_RKIND,2.386e6_RKIND,2.381e6_RKIND,2.377e6_RKIND,
     $     2.373e6_RKIND,2.368e6_RKIND,2.364e6_RKIND,2.360e6_RKIND,
     $     2.356e6_RKIND,2.352e6_RKIND,2.348e6_RKIND,2.344e6_RKIND,
     $     2.340e6_RKIND,2.336e6_RKIND,2.332e6_RKIND,2.328e6_RKIND,
     $     2.324e6_RKIND,2.320e6_RKIND,2.317e6_RKIND,2.313e6_RKIND,
     $     2.309e6_RKIND,2.306e6_RKIND,2.302e6_RKIND,2.299e6_RKIND,
     $     2.295e6_RKIND,2.292e6_RKIND,2.288e6_RKIND,2.285e6_RKIND,
     $     2.282e6_RKIND,2.278e6_RKIND,2.275e6_RKIND,2.272e6_RKIND,
     $     2.269e6_RKIND,2.266e6_RKIND,2.262e6_RKIND,2.259e6_RKIND,
     $     2.256e6_RKIND,2.253e6_RKIND,2.250e6_RKIND,2.247e6_RKIND,
     $     2.244e6_RKIND,2.242e6_RKIND,2.239e6_RKIND,2.236e6_RKIND,
     $     2.233e6_RKIND,2.230e6_RKIND,2.228e6_RKIND,2.225e6_RKIND,
     $     2.222e6_RKIND,2.219e6_RKIND,2.217e6_RKIND,2.214e6_RKIND,
     $     2.212e6_RKIND,2.209e6_RKIND,2.207e6_RKIND,2.204e6_RKIND,
     $     2.202e6_RKIND,2.199e6_RKIND,2.197e6_RKIND,2.194e6_RKIND,
     $     2.192e6_RKIND,2.189e6_RKIND,2.187e6_RKIND,2.185e6_RKIND,
     $     2.182e6_RKIND,2.180e6_RKIND,2.178e6_RKIND,2.176e6_RKIND,
     $     2.174e6_RKIND,2.171e6_RKIND,2.169e6_RKIND,2.167e6_RKIND,
     $     2.165e6_RKIND,2.163e6_RKIND,2.161e6_RKIND,2.159e6_RKIND,
     $     2.157e6_RKIND,2.155e6_RKIND,2.153e6_RKIND,2.151e6_RKIND,
     $     2.149e6_RKIND,2.147e6_RKIND,2.145e6_RKIND,2.143e6_RKIND,
     $     2.141e6_RKIND,2.139e6_RKIND,2.137e6_RKIND,2.135e6_RKIND,
     $     2.134e6_RKIND,2.132e6_RKIND,2.130e6_RKIND,2.128e6_RKIND,
     $     2.127e6_RKIND,2.125e6_RKIND,2.123e6_RKIND,2.122e6_RKIND,
     $     2.120e6_RKIND,2.118e6_RKIND,2.117e6_RKIND,2.115e6_RKIND,
     $     2.113e6_RKIND,2.112e6_RKIND,2.110e6_RKIND,2.109e6_RKIND,
     $     2.107e6_RKIND,2.106e6_RKIND,2.104e6_RKIND,2.103e6_RKIND,
     $     2.101e6_RKIND,2.100e6_RKIND,2.098e6_RKIND,2.097e6_RKIND,
     $     2.095e6_RKIND,2.094e6_RKIND,2.093e6_RKIND,2.091e6_RKIND,
     $     2.090e6_RKIND,2.088e6_RKIND,2.087e6_RKIND,2.086e6_RKIND,
     $     2.085e6_RKIND,2.083e6_RKIND,2.082e6_RKIND,2.081e6_RKIND,
     $     2.079e6_RKIND,2.078e6_RKIND,2.077e6_RKIND,2.076e6_RKIND,
     $     2.075e6_RKIND,2.073e6_RKIND,2.072e6_RKIND,2.071e6_RKIND,
     $     2.070e6_RKIND,2.069e6_RKIND,2.068e6_RKIND,2.066e6_RKIND,
     $     2.065e6_RKIND,2.064e6_RKIND,2.063e6_RKIND,2.062e6_RKIND,
     $     2.061e6_RKIND,2.060e6_RKIND,2.059e6_RKIND,2.058e6_RKIND,
     $     2.057e6_RKIND,2.056e6_RKIND,2.055e6_RKIND,2.054e6_RKIND,
     $     2.053e6_RKIND,2.052e6_RKIND,2.051e6_RKIND,2.050e6_RKIND,
     $     2.049e6_RKIND,2.048e6_RKIND,2.047e6_RKIND,2.046e6_RKIND,
     $     2.045e6_RKIND,2.044e6_RKIND,2.043e6_RKIND,2.042e6_RKIND,
     $     2.041e6_RKIND,2.041e6_RKIND,2.040e6_RKIND,2.039e6_RKIND,
     $     2.038e6_RKIND,2.037e6_RKIND,2.036e6_RKIND,2.035e6_RKIND,
     $     2.035e6_RKIND,2.034e6_RKIND,2.033e6_RKIND,2.032e6_RKIND,
     $     2.031e6_RKIND,2.030e6_RKIND,2.030e6_RKIND,2.029e6_RKIND,
     $     2.028e6_RKIND,2.027e6_RKIND,2.027e6_RKIND,2.026e6_RKIND,
     $     2.025e6_RKIND,2.024e6_RKIND,2.023e6_RKIND,2.023e6_RKIND,
     $     2.022e6_RKIND,2.021e6_RKIND,2.020e6_RKIND,2.020e6_RKIND,
     $     2.019e6_RKIND,2.018e6_RKIND,2.018e6_RKIND,2.017e6_RKIND,
     $     2.016e6_RKIND,2.015e6_RKIND,2.015e6_RKIND,2.014e6_RKIND,
     $     2.013e6_RKIND,2.013e6_RKIND,2.012e6_RKIND,2.011e6_RKIND,
     $     2.011e6_RKIND,2.010e6_RKIND,2.009e6_RKIND,2.009e6_RKIND,
     $     2.008e6_RKIND,2.007e6_RKIND,2.006e6_RKIND,2.006e6_RKIND,
     $     2.005e6_RKIND,2.004e6_RKIND,2.004e6_RKIND,2.003e6_RKIND,
     $     2.002e6_RKIND,2.002e6_RKIND,2.001e6_RKIND,2.001e6_RKIND,
     $     2.000e6_RKIND,1.999e6_RKIND,1.999e6_RKIND,1.998e6_RKIND,
     $     1.997e6_RKIND,1.997e6_RKIND,1.996e6_RKIND,1.995e6_RKIND,
     $     1.995e6_RKIND,1.994e6_RKIND,1.993e6_RKIND,1.993e6_RKIND,
     $     1.992e6_RKIND,1.991e6_RKIND,1.991e6_RKIND,1.990e6_RKIND,
     $     1.989e6_RKIND,1.989e6_RKIND,1.988e6_RKIND,1.987e6_RKIND,
     $     1.987e6_RKIND,1.986e6_RKIND,1.986e6_RKIND,1.985e6_RKIND,
     $     1.984e6_RKIND,1.984e6_RKIND,1.983e6_RKIND,1.982e6_RKIND,
     $     1.982e6_RKIND,1.981e6_RKIND,1.980e6_RKIND,1.980e6_RKIND,
     $     1.979e6_RKIND,1.978e6_RKIND,1.977e6_RKIND,1.977e6_RKIND,
     $     1.976e6_RKIND,1.975e6_RKIND,1.975e6_RKIND,1.974e6_RKIND,
     $     1.973e6_RKIND,1.973e6_RKIND,1.972e6_RKIND,1.971e6_RKIND,
     $     1.970e6_RKIND,1.970e6_RKIND,1.969e6_RKIND,1.968e6_RKIND,
     $     1.968e6_RKIND,1.967e6_RKIND,1.966e6_RKIND,1.965e6_RKIND,
     $     1.965e6_RKIND,1.964e6_RKIND,1.963e6_RKIND,1.962e6_RKIND,
     $     1.962e6_RKIND,1.961e6_RKIND,1.960e6_RKIND,1.959e6_RKIND,
     $     1.959e6_RKIND,1.958e6_RKIND,1.957e6_RKIND,1.956e6_RKIND,
     $     1.956e6_RKIND,1.955e6_RKIND,1.954e6_RKIND,1.953e6_RKIND,
     $     1.953e6_RKIND,1.952e6_RKIND,1.951e6_RKIND,1.950e6_RKIND,
     $     1.950e6_RKIND,1.949e6_RKIND,1.948e6_RKIND,1.947e6_RKIND,
     $     1.946e6_RKIND,1.946e6_RKIND,1.945e6_RKIND,1.944e6_RKIND,
     $     1.943e6_RKIND,1.943e6_RKIND,1.942e6_RKIND,1.941e6_RKIND,
     $     1.940e6_RKIND,1.940e6_RKIND,1.939e6_RKIND,1.938e6_RKIND,
     $     1.937e6_RKIND,1.936e6_RKIND,1.936e6_RKIND,1.935e6_RKIND,
     $     1.934e6_RKIND,1.933e6_RKIND,1.933e6_RKIND,1.932e6_RKIND,
     $     1.931e6_RKIND,1.930e6_RKIND,1.929e6_RKIND,1.929e6_RKIND,
     $     1.928e6_RKIND,1.927e6_RKIND,1.926e6_RKIND,1.926e6_RKIND,
     $     1.925e6_RKIND,1.924e6_RKIND,1.923e6_RKIND,1.922e6_RKIND,
     $     1.922e6_RKIND,1.921e6_RKIND,1.920e6_RKIND,1.919e6_RKIND,
     $     1.919e6_RKIND,1.918e6_RKIND,1.917e6_RKIND,1.916e6_RKIND,
     $     1.916e6_RKIND,1.915e6_RKIND,1.914e6_RKIND,1.913e6_RKIND,
     $     1.913e6_RKIND,1.912e6_RKIND,1.911e6_RKIND,1.910e6_RKIND,
     $     1.910e6_RKIND,1.909e6_RKIND,1.908e6_RKIND,1.908e6_RKIND,
     $     1.907e6_RKIND,1.906e6_RKIND,1.905e6_RKIND,1.905e6_RKIND,
     $     1.904e6_RKIND,1.903e6_RKIND,1.903e6_RKIND,1.902e6_RKIND,
     $     1.901e6_RKIND,1.900e6_RKIND,1.900e6_RKIND,1.899e6_RKIND,
     $     1.898e6_RKIND,1.898e6_RKIND,1.897e6_RKIND,1.896e6_RKIND,
     $     1.896e6_RKIND,1.895e6_RKIND,1.894e6_RKIND,1.894e6_RKIND,
     $     1.893e6_RKIND,1.892e6_RKIND,1.892e6_RKIND,1.891e6_RKIND,
     $     1.890e6_RKIND,1.890e6_RKIND,1.889e6_RKIND,1.888e6_RKIND,
     $     1.888e6_RKIND,1.887e6_RKIND,1.886e6_RKIND,1.886e6_RKIND,
     $     1.885e6_RKIND,1.884e6_RKIND,1.884e6_RKIND,1.883e6_RKIND,
     $     1.883e6_RKIND,1.882e6_RKIND,1.881e6_RKIND,1.881e6_RKIND,
     $     1.880e6_RKIND,1.880e6_RKIND,1.879e6_RKIND,1.878e6_RKIND,
     $     1.878e6_RKIND,1.877e6_RKIND,1.877e6_RKIND,1.876e6_RKIND,
     $     1.875e6_RKIND,1.875e6_RKIND,1.874e6_RKIND,1.874e6_RKIND,
     $     1.873e6_RKIND,1.873e6_RKIND,1.872e6_RKIND,1.871e6_RKIND,
     $     1.871e6_RKIND,1.870e6_RKIND,1.870e6_RKIND,1.869e6_RKIND,
     $     1.869e6_RKIND,1.868e6_RKIND,1.868e6_RKIND,1.867e6_RKIND,
     $     1.867e6_RKIND,1.866e6_RKIND,1.865e6_RKIND,1.865e6_RKIND,
     $     1.864e6_RKIND,1.864e6_RKIND,1.863e6_RKIND,1.863e6_RKIND,
     $     1.862e6_RKIND,1.862e6_RKIND,1.861e6_RKIND,1.861e6_RKIND,
     $     1.860e6_RKIND,1.860e6_RKIND,1.859e6_RKIND,1.859e6_RKIND,
     $     1.858e6_RKIND,1.858e6_RKIND,1.857e6_RKIND,1.857e6_RKIND,
     $     1.856e6_RKIND,1.856e6_RKIND,1.855e6_RKIND,1.855e6_RKIND,
     $     1.854e6_RKIND,1.854e6_RKIND,1.853e6_RKIND,1.853e6_RKIND,
     $     1.853e6_RKIND,1.852e6_RKIND,1.852e6_RKIND,1.851e6_RKIND,
     $     1.851e6_RKIND,1.850e6_RKIND,1.850e6_RKIND,1.849e6_RKIND,
     $     1.849e6_RKIND,1.848e6_RKIND,1.848e6_RKIND,1.848e6_RKIND,
     $     1.847e6_RKIND,1.847e6_RKIND,1.846e6_RKIND,1.846e6_RKIND,
     $     1.845e6_RKIND,1.845e6_RKIND,1.845e6_RKIND,1.844e6_RKIND,
     $     1.844e6_RKIND,1.843e6_RKIND,1.843e6_RKIND,1.843e6_RKIND,
     $     1.842e6_RKIND,1.842e6_RKIND,1.841e6_RKIND,1.841e6_RKIND,
     $     1.841e6_RKIND,1.840e6_RKIND,1.840e6_RKIND,1.839e6_RKIND,
     $     1.839e6_RKIND,1.839e6_RKIND,1.838e6_RKIND,1.838e6_RKIND,
     $     1.837e6_RKIND,1.837e6_RKIND,1.837e6_RKIND,1.836e6_RKIND,
     $     1.836e6_RKIND,1.836e6_RKIND,1.835e6_RKIND,1.835e6_RKIND,
     $     1.834e6_RKIND,1.834e6_RKIND,1.834e6_RKIND,1.833e6_RKIND,
     $     1.833e6_RKIND,1.833e6_RKIND,1.832e6_RKIND,1.832e6_RKIND,
     $     1.832e6_RKIND,1.831e6_RKIND,1.831e6_RKIND,1.831e6_RKIND,
     $     1.830e6_RKIND,1.830e6_RKIND,1.830e6_RKIND,1.829e6_RKIND,
     $     1.829e6_RKIND,1.829e6_RKIND,1.828e6_RKIND,1.828e6_RKIND,
     $     1.828e6_RKIND,1.827e6_RKIND,1.827e6_RKIND,1.827e6_RKIND,
     $     1.826e6_RKIND,1.826e6_RKIND,1.826e6_RKIND,1.826e6_RKIND,
     $     1.825e6_RKIND,1.825e6_RKIND,1.825e6_RKIND,1.824e6_RKIND,
     $     1.824e6_RKIND,1.824e6_RKIND,1.824e6_RKIND,1.823e6_RKIND,
     $     1.823e6_RKIND,1.823e6_RKIND,1.822e6_RKIND,1.822e6_RKIND,
     $     1.822e6_RKIND,1.822e6_RKIND,1.821e6_RKIND,1.821e6_RKIND,
     $     1.821e6_RKIND,1.820e6_RKIND,1.820e6_RKIND,1.820e6_RKIND,
     $     1.820e6_RKIND,1.819e6_RKIND,1.819e6_RKIND,1.819e6_RKIND,
     $     1.819e6_RKIND,1.818e6_RKIND,1.818e6_RKIND,1.818e6_RKIND,
     $     1.818e6_RKIND,1.817e6_RKIND,1.817e6_RKIND,1.817e6_RKIND,
     $     1.817e6_RKIND,1.816e6_RKIND,1.816e6_RKIND,1.816e6_RKIND,
     $     1.816e6_RKIND,1.815e6_RKIND,1.815e6_RKIND,1.815e6_RKIND,
     $     1.815e6_RKIND,1.815e6_RKIND,1.814e6_RKIND,1.814e6_RKIND,
     $     1.814e6_RKIND,1.814e6_RKIND,1.813e6_RKIND,1.813e6_RKIND,
     $     1.813e6_RKIND,1.813e6_RKIND,1.813e6_RKIND,1.812e6_RKIND,
     $     1.812e6_RKIND,1.812e6_RKIND,1.812e6_RKIND,1.812e6_RKIND,
     $     1.811e6_RKIND,1.811e6_RKIND,1.811e6_RKIND,1.811e6_RKIND,
     $     1.811e6_RKIND,1.810e6_RKIND,1.810e6_RKIND,1.810e6_RKIND,
     $     1.810e6_RKIND,1.810e6_RKIND,1.810e6_RKIND,1.809e6_RKIND,
     $     1.809e6_RKIND,1.809e6_RKIND,1.809e6_RKIND,1.809e6_RKIND,
     $     1.808e6_RKIND,1.808e6_RKIND,1.808e6_RKIND,1.808e6_RKIND,
     $     1.808e6_RKIND,1.808e6_RKIND,1.807e6_RKIND,1.807e6_RKIND,
     $     1.807e6_RKIND,1.807e6_RKIND,1.807e6_RKIND,1.807e6_RKIND,
     $     1.807e6_RKIND,1.806e6_RKIND,1.806e6_RKIND,1.806e6_RKIND,
     $     1.806e6_RKIND,1.806e6_RKIND,1.806e6_RKIND,1.805e6_RKIND,
     $     1.805e6_RKIND,1.805e6_RKIND,1.805e6_RKIND,1.805e6_RKIND,
     $     1.805e6_RKIND,1.805e6_RKIND,1.805e6_RKIND,1.804e6_RKIND,
     $     1.804e6_RKIND,1.804e6_RKIND,1.804e6_RKIND,1.804e6_RKIND,
     $     1.804e6_RKIND,1.804e6_RKIND,1.803e6_RKIND,1.803e6_RKIND,
     $     1.803e6_RKIND,1.803e6_RKIND,1.803e6_RKIND,1.803e6_RKIND,
     $     1.803e6_RKIND,1.803e6_RKIND,1.803e6_RKIND,1.802e6_RKIND,
     $     1.802e6_RKIND,1.802e6_RKIND,1.802e6_RKIND,1.802e6_RKIND,
     $     1.802e6_RKIND,1.802e6_RKIND,1.802e6_RKIND,1.802e6_RKIND,
     $     1.801e6_RKIND,1.801e6_RKIND,1.801e6_RKIND,1.801e6_RKIND,
     $     1.801e6_RKIND,1.801e6_RKIND,1.801e6_RKIND,1.801e6_RKIND,
     $     1.801e6_RKIND,1.801e6_RKIND,1.801e6_RKIND,1.800e6_RKIND,
     $     1.800e6_RKIND,1.800e6_RKIND,1.800e6_RKIND,1.800e6_RKIND,
     $     1.800e6_RKIND,1.800e6_RKIND,1.800e6_RKIND,1.800e6_RKIND,
     $     1.800e6_RKIND,1.800e6_RKIND,1.800e6_RKIND,1.800e6_RKIND,
     $     1.799e6_RKIND,1.799e6_RKIND,1.799e6_RKIND,1.799e6_RKIND,
     $     1.799e6_RKIND,1.799e6_RKIND,1.799e6_RKIND,1.799e6_RKIND,
     $     1.799e6_RKIND,1.799e6_RKIND,1.799e6_RKIND,1.799e6_RKIND,
     $     1.799e6_RKIND,1.799e6_RKIND,1.799e6_RKIND,1.799e6_RKIND,
     $     1.798e6_RKIND,1.798e6_RKIND,1.798e6_RKIND,1.798e6_RKIND,
     $     1.798e6_RKIND,1.798e6_RKIND,1.798e6_RKIND,1.798e6_RKIND,
     $     1.798e6_RKIND,1.798e6_RKIND,1.798e6_RKIND,1.798e6_RKIND,
     $     1.798e6_RKIND,1.798e6_RKIND,1.798e6_RKIND,1.798e6_RKIND,
     $     1.798e6_RKIND,1.798e6_RKIND,1.798e6_RKIND,1.798e6_RKIND,
     $     1.798e6_RKIND,1.798e6_RKIND,1.798e6_RKIND,1.797e6_RKIND,
     $     1.797e6_RKIND,1.797e6_RKIND,1.797e6_RKIND,1.797e6_RKIND,
     $     1.797e6_RKIND,1.797e6_RKIND,1.797e6_RKIND,1.797e6_RKIND,
     $     1.797e6_RKIND,1.797e6_RKIND,1.797e6_RKIND,1.797e6_RKIND,
     $     1.797e6_RKIND,1.797e6_RKIND,1.797e6_RKIND,1.797e6_RKIND,
     $     1.797e6_RKIND,1.797e6_RKIND,1.797e6_RKIND,1.797e6_RKIND,
     $     1.797e6_RKIND,1.797e6_RKIND,1.797e6_RKIND,1.797e6_RKIND,
     $     1.797e6_RKIND,1.797e6_RKIND,1.797e6_RKIND,1.797e6_RKIND,
     $     1.797e6_RKIND,1.797e6_RKIND,1.797e6_RKIND,1.797e6_RKIND,
     $     1.797e6_RKIND,1.797e6_RKIND,1.797e6_RKIND,1.797e6_RKIND,
     $     1.797e6_RKIND,1.797e6_RKIND,1.797e6_RKIND,1.797e6_RKIND,
     $     1.797e6_RKIND,1.797e6_RKIND,1.797e6_RKIND,1.797e6_RKIND,
     $     1.797e6_RKIND,1.797e6_RKIND,1.797e6_RKIND,1.797e6_RKIND,
     $     1.797e6_RKIND,1.797e6_RKIND,1.797e6_RKIND,1.797e6_RKIND,
     $     1.797e6_RKIND,1.797e6_RKIND,1.797e6_RKIND,1.797e6_RKIND,
     $     1.797e6_RKIND,1.797e6_RKIND,1.797e6_RKIND,1.797e6_RKIND,
     $     1.797e6_RKIND,1.797e6_RKIND,1.797e6_RKIND,1.797e6_RKIND,
     $     1.797e6_RKIND,1.797e6_RKIND,1.797e6_RKIND,1.797e6_RKIND,
     $     1.797e6_RKIND,1.797e6_RKIND,1.797e6_RKIND,1.797e6_RKIND,
     $     1.797e6_RKIND,1.797e6_RKIND,1.797e6_RKIND,1.797e6_RKIND,
     $     1.797e6_RKIND,1.797e6_RKIND,1.797e6_RKIND,1.797e6_RKIND,
     $     1.797e6_RKIND,1.797e6_RKIND,1.797e6_RKIND,1.797e6_RKIND,
     $     1.797e6_RKIND,1.797e6_RKIND,1.797e6_RKIND,1.798e6_RKIND,
     $     1.798e6_RKIND,1.798e6_RKIND,1.798e6_RKIND,1.798e6_RKIND,
     $     1.798e6_RKIND,1.798e6_RKIND,1.798e6_RKIND,1.798e6_RKIND,
     $     1.798e6_RKIND,1.798e6_RKIND,1.798e6_RKIND,1.798e6_RKIND,
     $     1.798e6_RKIND,1.798e6_RKIND,1.798e6_RKIND,1.798e6_RKIND,
     $     1.798e6_RKIND,1.798e6_RKIND,1.798e6_RKIND,1.798e6_RKIND,
     $     1.798e6_RKIND,1.798e6_RKIND,1.798e6_RKIND,1.798e6_RKIND,
     $     1.798e6_RKIND,1.798e6_RKIND,1.798e6_RKIND,1.798e6_RKIND,
     $     1.798e6_RKIND,1.798e6_RKIND,1.798e6_RKIND,1.798e6_RKIND,
     $     1.799e6_RKIND,1.799e6_RKIND,1.799e6_RKIND,1.799e6_RKIND,
     $     1.799e6_RKIND,1.799e6_RKIND,1.799e6_RKIND,1.799e6_RKIND,
     $     1.799e6_RKIND,1.799e6_RKIND,1.799e6_RKIND,1.799e6_RKIND,
     $     1.799e6_RKIND,1.799e6_RKIND,1.799e6_RKIND,1.799e6_RKIND,
     $     1.799e6_RKIND,1.799e6_RKIND,1.799e6_RKIND,1.799e6_RKIND,
     $     1.799e6_RKIND,1.799e6_RKIND,1.799e6_RKIND,1.799e6_RKIND,
     $     1.799e6_RKIND,1.799e6_RKIND,1.799e6_RKIND,1.800e6_RKIND,
     $     1.800e6_RKIND,1.800e6_RKIND,1.800e6_RKIND,1.800e6_RKIND,
     $     1.800e6_RKIND,1.800e6_RKIND,1.800e6_RKIND,1.800e6_RKIND,
     $     1.800e6_RKIND,1.800e6_RKIND,1.800e6_RKIND,1.800e6_RKIND/

      if (mass .lt. minmass) then
         luminosity = lumtable(1)
         lifetime = lifetable(1)
      elseif (mass .gt. maxmass) then
         luminosity = lumtable(995)
         lifetime = lifetable(995)
      else
         luminosity = lumtable(nint(mass-minmass,IKIND))
         lifetime = lifetable(nint(mass-minmass,IKIND))
      endif

      return

      end
